#! /usr/bin/env python
'''
Module Name:    TaskMgr.py

Description:     This is the main task manager for MUS.

Copyright:      Toshiba Confidential

                OCO Source Materials

                Copyright (C) 2012,2013 Toshiba Global Commerce Solutions, Inc.

                The source code for this program is not published
                or otherwise divested of its trade secrets,
                irrespective of what has been deposited with the
                U. S. Copyright Office.

Initial Author: Francisco Liwa

Function:

Notes:

Dependencies:    None

Restrictions:    None

Last Modified: 12 December 2013

'''
#importing core modules
import os
import sys
import subprocess
import time

#importing helper modules 

#from monitorupgrade import MUSConfigParser
from monitorupgrade import TLog as TLOG
from monitorupgrade import boot_update as bu
from monitorupgrade import Configs as Configs
from monitorupgrade import Utils as utils
from monitorupgrade import SampoVideoFwFlash as sampo
from monitorupgrade import EloTouchFwFlash as elo
from monitorupgrade import OcularTouchFwFlash as ocular
from monitorupgrade import WistronVideoFwFlash as wistron


#Globals
FILE_FOUND = 1
FILE_NOT_FOUND = 0
EXIT_SUCCESS = 0
EXIT_FAILURE = 1 
UNKNOWN = -1
NO_MONITORS_ATTACHED = 0

#Error Codes
ERROR_CODE_DEVICE_NOT_EXIST = 4
ERROR_CODE_OPERATION_FAILED = 11
ERROR_CODE_INVALID_PARAM = 21

#Error Strings
ERROR_STR_DEVICE_NOT_EXIST = 'Device not exist'
ERROR_STR_OPERATION_FAILED = 'Operation failed'
ERROR_STR_INVALID_PARAM = 'Invalid Parameter'
ERROR_STR_Runtime = 'Runtime error'


#sampo stuffs here
SAMPO_UTIL = '/opt/toshiba/mus/op_utils/sampo/4820MonUtil'
AFTER_LOCAL = '/etc/init.d/after.local'
RECOVERY_STR = '4820MonUtil'

#ELO Stuffs here    
ELO_DOWNLOAD = '/opt/toshiba/mus/op_utils/elo/EloDownload.sh'
ELO_CONFIG = '/opt/toshiba/mus/op_utils/elo/EloConfig.sh'
ELO_RECOVERY_FILE = '/opt/toshiba/mus/op_utils/elo/recovery.txt'
ELO_INI = '/opt/toshiba/mus/UpdateTasks/TouchConfig/EloTouch/toshiba-mus2.0_up_2xx5xx_touch-config_elo/eloConfigOpt.ini'

#DWU
AIP_RECOVERY_FILE = '/opt/toshiba/mus/op_utils/aipfwupdate/recovery.txt'
OCULAR_RECOVERY_FILE = '/opt/toshiba/mus/op_utils/mxtapp/recovery.txt'

#helpers
LSTMON = '/usr/bin/lstmon2'
RSTELO = '/usr/bin/rstelo2'
CHKELOPATCH = '/usr/bin/chk_elo_patch'
CHKFOUSB = '/usr/bin/chk_fousb'

#Hub vendors
CYPRESS_VID = '04b4'
SMC_VID = '0424'

#Product vendors
IBM_VID = '04b3'
ELO_VID = '04e7'
TGCS_VID = '0f66'
OCULAR_VID = '03eb'

PAD = ' '
SERIAL_LEN = 20

#Component ID;s
DWU_COMPONENTS = ['mcu',
                  'hw_cfg',
                  'scalar',
                  'capsense',
                  ]

UP_RPMS_DIR = '/opt/toshiba/mus/RpmUpdates'


UPDATE_TYPES = ['YAM_VIDEO_FW_UPDATE',   #0
                'ANK_VIDEO_FW_UPDATE',   #1
                'YAM_TOUCH_FW_UPDATE',   #2
                'ANK_TOUCH_FW_UPDATE',   #3
                'YNK_TOUCH_CONFIG',       #4
                'DWU_VIDEO_FW_UPDATE',   #5
                'DWU_TOUCH_FW_UPDATE',   #6
                'DWU_TOUCH_CONFIG',       #7
                '3MD_TOUCH_FW_UPDATE',   #8
                '3MD_TOUCH_CONFIG',       #9
                'SPECIAL_TASK'            #10   
                ]


                
#======================================================== Main Task Manager Class ======================================================        


#==========================================================================================================
'''
TaskMgr - The main class to handle and call other helper classes for individual task
'''
#==========================================================================================================        
class TaskMgr:
    init = False
    fw_path=''
    fw_name=''
    conf_path=''
    controller=''
    retry=0
    RECOVERY_MODE = False
    HAS_4820_MON = False
    mon_list = []
    video_up_tasks = []
    touch_up_tasks = []
    pending_tasks = []
    rcv_cmd = ''
    
    def __init__(self):
        '''
        Constructor
        '''
        #print "TaskMgr Constructed"
        TLOG.logger.info('TaskMgr Constructed!')

#==========================================================================================================
# initialize() - function to check if task manager has already been initialized
#==========================================================================================================
    def initialize(self):
        
        if (self.init):
            #print'TaskMgr::initialize-Already Initialized'
            TLOG.logger.info('TaskMgr::initialize-Already Initialized')
        else:
            self.init = True
            #print'TaskMgr::initialize-Initialized OK'
            self.runRpmUpdates()
            TLOG.logger.info('TaskMgr::initialize-Initialized OK')
            

#==========================================================================================================
# checkFousb() - Check if elofousb KMP driver module is installed or not
#              - if installed then exit MUS 
#              - This is for MUS 1.0 only, MUS 1.1 don't need it
#==========================================================================================================
    def checkFousb(self):
        
        res = -1
        process = subprocess.Popen(CHKFOUSB, shell=True, stdout=subprocess.PIPE)
        while True:
            line = process.stdout.readline()
            #print line
            if not line:
                break
            #TLOG.logger.info(line)
            res = int ( line.strip('\n') )    
        process.wait()
        return int(res)


#==========================================================================================================
# checkEloUsbPatch() - Checks whether elousb driver with fw_quirk adjustment is installed or not
#                    - If not installed  then exit MUS
#                    - This is to ensure we don't have complications with 'broken firmware' workaround
#==========================================================================================================
    def checkEloUsbPatch(self):
        
        res = -1
        process = subprocess.Popen(CHKELOPATCH, shell=True, stdout=subprocess.PIPE)
        while True:
            line = process.stdout.readline()
            #print line
            if not line:
                break
            #TLOG.logger.info(line)
            res = int( line.strip('\n') )    
        process.wait()
        return res

#==========================================================================================================
# doRstElo()   - Reset EloUSB devices
#==========================================================================================================
    def doRstElo(self):
        
        #res = -1
        process = subprocess.Popen(RSTELO, shell=True, stdout=subprocess.PIPE)
        while True:
            line = process.stdout.readline()
            #print line
            if not line:
                break
            #TLOG.logger.info(line)
            #res = int ( line.strip('\n') )    
        process.wait()
    
        #return int(res)

#==========================================================================================================
# runRpmUpdates() - functions to check whether we have new UP rpm files in the rpmUpdate directory
#==========================================================================================================
    def runRpmUpdates(self):
        bu.process()
#==========================================================================================================
# getMonitorList() - function to get the list of all attached monitors and its details --Video/Touch FW, etc
#==========================================================================================================
    def getMonitorList(self):
        process = subprocess.Popen('lstmon2', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE) 
        while True:
            output_line = process.stdout.readline().strip('\n')
            if not output_line:
                break
            hub,entity,vid_pid,bcdDevice,controller = output_line.split(' ')
            self.mon_list.append((hub,entity,vid_pid,bcdDevice,controller ))                       
            
            process.wait()
        return len(self.mon_list)


 
#==========================================================================================================
# getMonitorList2() - function to get the list of all attached monitors and its details --Video/Touch FW, etc
#==========================================================================================================
    def getMonitorList2(self):
        process = subprocess.Popen('lstmon2 >> /tmp/monlist.txt', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        process.wait()                  
    
        #mon_list = []
        f = open('/tmp/monlist.txt', 'r+')
        lines = f.readlines()
        f.close() 
       
        process = subprocess.Popen('rm /tmp/monlist.txt ', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        process.wait() 
        
        curr_id = 0
        for line in lines:
            if ( "TDV" in line ):
                line = line.strip('\n')
                #hub_id, entity, vid_pid, bcdDevice, usb_dev_id,serial_1, serial_2, serial_3 = line.split(' ')
                tokens = line.split(' ')
                hub_id = tokens[0]
                entity = tokens[1]
                vid_pid = tokens[2]
                bcdDevice = tokens[3]
                usb_dev_id = tokens[4]
                serial =  tokens[5] 
                #print hub_id,entity,vid_pid,bcdDevice,controller1,controller2,controller3
                
            elif ( "ADT" in line ):
                line = line.strip('\n')
                #hub_id, entity, vid_pid, bcdDevice, usb_dev_id,serial_1, serial_2, serial_3 = line.split(' ')
                tokens = line.split(' ')
                hub_id = tokens[0]
                entity = tokens[1]
                vid_pid = tokens[2]
                bcdDevice = tokens[3]
                usb_dev_id = tokens[4]
                serial = tokens[5]

                #print hub_id,entity,vid_pid,bcdDevice,controller1,controller2,controller3
            else:
                line = line.strip('\n')
                hub_id, entity, vid_pid, bcdDevice, usb_dev_id, serial= line.split(' ')
                #print hub_id,entity,vid_pid,bcdDevice,controller
    
            if (hub_id[-1:] != curr_id):
     
                curr_id = hub_id[-1:]
              
                mon_ = TouchMonitor()
                mon_.setScreenNo(curr_id)
    
                if entity == 'TDV' or entity == 'SYV' or entity == 'SAV':
                   
                    mon_.setVideoFwLevel(bcdDevice)
                    mon_.setVideoSerialNumber(serial)
                    mon_.setVideoVidPid(vid_pid)
                    mon_.setVideoDevId(usb_dev_id)
                    
                    if entity == 'TDV':
                        mon_.setMonType('DWU')
                    elif entity == 'SYV':
                        mon_.setMonType('YAM')
                        self.HAS_4820_MON = True
                    elif entity == 'SAV':
                        mon_.setMonType('ANK')
                        self.HAS_4820_MON = True
                
                elif entity =='ADT' or entity == 'EKT' or entity == 'AIT':
                    
                    l = len(serial)
                    print 'serial_len:' + str(l)
                    if ( entity =='ADT' or entity == 'AIT') :
                        if l < SERIAL_LEN:
                            pad_count = SERIAL_LEN - l
                            for i in range(pad_count):
                                serial += PAD
                                
                                 
                    mon_.setTouchSerialNumber(serial)
                    mon_.setTouchFwLevel(bcdDevice)
                    mon_.setTouchVidPid(vid_pid)
                    mon_.setTouchDevId(usb_dev_id)
                    
                    #print entity,vid_pid,bcdDevice,serial
               
            elif (hub_id[-1:] == curr_id):
            
                if entity == 'TDV' or entity == 'SYV' or entity == 'SAV':
               
                    mon_.setVideoFwLevel(bcdDevice)
                    mon_.setVideoSerialNumber(serial)
                    mon_.setVideoVidPid(vid_pid)
                    mon_.setVideoDevId(usb_dev_id)
                    
                    if entity == 'TDV':
                        mon_.setMonType('DWU')
                    elif entity == 'SYV':
                        mon_.setMonType('YAM')
                        self.HAS_4820_MON = True
                    elif entity == 'SAV':
                        mon_.setMonType('ANK')
                        self.HAS_4820_MON = True
    
                elif entity =='ADT' or entity == 'EKT':
                    
                    l = len(serial)
                    #print 'serial_len:' + str(l)
                    if ( entity =='ADT' ) :
                        if l < SERIAL_LEN:
                            pad_count = SERIAL_LEN - l
                            for i in range(pad_count):
                                serial += PAD
                    
                    mon_.setTouchFwLevel(bcdDevice) 
                    mon_.setTouchSerialNumber(serial)
                    mon_.setTouchVidPid(vid_pid)
                    mon_.setTouchDevId(usb_dev_id)
                    
            
                self.mon_list.append(mon_)
        
        print '#_of_monitors attached: ' + str(len(self.mon_list))
        return self.mon_list
           

#==========================================================================================================
# parseFirmwareLevel() - returns the firmware level embedded on each update folders
#==========================================================================================================   
    def parseFirmwareLevel(self,rpm):
    
        level_str =''
        parts = rpm.split("_")
        num_parts = len(parts)
        if (num_parts>0):
            pattern = parts[0]
        if (num_parts>1):
            mus_rpm_type = parts[1]
        if (num_parts>2):
            target_models = parts[2]
        if (num_parts>3):
            update_type = parts[3]
        if (num_parts>4):
            tmp = parts[4]
            endparts = tmp.split("-")
            num_endparts = len(endparts)
            if (num_endparts>0):
                level_str = endparts[0]
        #print ' FW LEVEL:', level_str
        TLOG.logger.info('TaskMgr::parseFirmwareLevel() - FW LEVEL: ' + level_str)
                
        count = len(level_str)
        fw_level = level_str[count-2:]    
        return fw_level

#==========================================================================================================
# checkForTouchFwUpdate() - check whether we have necessary Touch Firmware Update
#                         - if there are then it will update the pending tasks list
#==========================================================================================================    
    def  checkForTouchFwUpdate(self,vid_pid,fw,controller,mon_type,devid):
        
        vid,pid = vid_pid.split(':') 
        touch_fw = fw[2:]
        
        # Elo Touch
        if vid == ELO_VID:
            print '[Touch Firmware]'
            TLOG.logger.info('TaskMgr::checkForVideoFwUpdate() -[Touch Firmware]')
            serial = controller[2:]
            if (pid == "0009" or pid == "0030"):
            #Check current to see if this is the Touch for the Current Monitor
                if (mon_type == 'YAM'):
                    self.updatePendingTaskList('EloTouchFwFlashTask',touch_fw,serial,'/opt/toshiba/mus/UpdateTasks/Touch/YAM')
                elif (mon_type == 'ANK'):
                    self.updatePendingTaskList('EloTouchFwFlashTask',touch_fw,serial,'/opt/toshiba/mus/UpdateTasks/Touch/ANK')
        # Dawu Touch Atmel/Ocular
        elif ( vid  == OCULAR_VID ): 
                    print '[Touch Firmware]'
                    TLOG.logger.info('TaskMgr::checkForTouchFwUpdate() -[Touch Firmware]')
                    
                    serial = controller[2:]
                    fw_path = '/opt/toshiba/mus/UpdateTasks/Touch/DWU'
                       
                    if os.path.exists(fw_path):                 
                        self.updatePendingDWUTouchTasks('OcularTouchFwFlashTask',touch_fw,devid,fw_path)                    
                                
                
    
#==========================================================================================================
# checkForVideoFwUpdate() - check whether we have necessary Video Firmware Update
#                         - if there are then it will update the pending tasks list
#==========================================================================================================    
    def  checkForVideoFwUpdate(self,vid_pid,fw,controller,mon_type,devid):
        
        #separate the VID , PID values
        vid,pid = vid_pid.split(':') 
        
        #YANAK
        if (vid == IBM_VID):
            video_fw = fw[2:]
            
            print '[Video Firmware]'
            TLOG.logger.info('TaskMgr::checkForVideoFwUpdate() -[Video Firmware]')
            serial,model = controller.split(';')
            serial = serial[2:]
            if (mon_type == 'YAM'):  # Found Yam Monitor
                self.updatePendingTaskList('SampoVideoFwFlashTask',video_fw,serial,'/opt/toshiba/mus/UpdateTasks/Video/YAM')
            elif (mon_type == 'ANK'):  # Found Ank Monitor
                self.updatePendingTaskList('SampoVideoFwFlashTask',video_fw,serial,'/opt/toshiba/mus/UpdateTasks/Video/ANK')
        #DAWU             
        elif (vid == TGCS_VID):    
            print '[Video Firmware]'
            TLOG.logger.info('TaskMgr::checkForVideoFwUpdate() -[Video Firmware]')
            
            serial = controller
            fws = fw.split(',')
            for idx ,comp_fw in enumerate(fws) :
                print 'FW_Component_'+ str (idx)
                comp_fw = comp_fw[2:]
                fw_path = '/opt/toshiba/mus/UpdateTasks/Video/DWU/'
                #fw_path = '/opt/toshiba/mus/UpdateTasks/Video/DWU/'+ DWU_COMPONENTS[idx]
                if os.path.exists(fw_path):
                    self.updatePendingDWUVideoTasks('WistronVideoFwFlashTask',comp_fw,serial,fw_path,DWU_COMPONENTS[idx])

#==========================================================================================================
# updatePendingDWUVideoTasks() - update the PendingTasks list based on the firmware we got
#==========================================================================================================    
    
    def updatePendingDWUTouchTasks(self,task_name,fw,serial,path):
        from os import walk
        f = []
        dirpath = ''
        dirnames = ''
        filenames = ''
        name = ''
        
        # get a list of all the folders inside the UpdateTask Folder
        for (dirpath,dirnames,filenames) in walk(path):
            f.extend(filenames)
            break
        
        if fw.isdigit() == False:
            #for firmware levels with bcd values, we convert them to hex
            fw = int(fw,16)         
        
        # keep a record of old firmware value
        current_fw = int(fw)            
        
        max_level = 0
        #iterate thru the list of firmware folders
        folder_str  =''
        for folder in dirnames:   
            folder_name = folder
            folder=self.parseFirmwareLevel(folder)
            
            # hold the string representation of the folder name
            folder_str = folder               
                         
            #compare the firmware of the device to the folder name 
            #corresponding to available firmware updates            
            if int(folder) > max_level:
                max_level = int(folder)
                
                #assign the new folder location
                new_fw_str = folder_str       
                name = folder_name
                print "Found! FW Level:", new_fw_str
                TLOG.logger.info('TaskMgr::updatePendingDWUTouchTasks() - Found! FW package name:' + new_fw_str)
                TLOG.logger.info('TaskMgr::updatePendingDWUTouchTasks() - Found! FW package name:' + name)

        fw_path = (os.path.join(dirpath,name))
        
        tskcfg = Configs.UpdateTaskConfig(fw_path)
        fw = tskcfg.getFirmwareLevel()
        fw = fw[2:]
        xcfg = tskcfg.getXcfgLevel()
                
        print int(fw)
        if ( int(fw) > current_fw) :
            self.touch_up_tasks.append( (task_name,fw_path,serial) )
        
        #Load only configuration. This is if there are same FW level but different configuration    
        elif ( int(fw) == current_fw) :
            task_name = 'OcularTouchXcfgLoadTask'
            self.touch_up_tasks.append( (task_name,fw_path,serial) )            
        
        else:
            print 'No Firmware/XCFG Update Available!'  
            TLOG.logger.info('TaskMgr::updatePendingDWUTouchTasks() - Firmware/XCFG Update Available!')
                                                                                              
                                                             

#==========================================================================================================
# updatePendingDWUVideoTasks() - update the PendingTasks list based on the firmware we got
#==========================================================================================================    
    
    def updatePendingDWUVideoTasks(self,task_name,fw,serial,path,comp):
        from os import walk
        f = []
        dirpath = ''
        dirnames = ''
        filenames = ''
        name = ''
        
        # get a list of all the folders inside the UpdateTask Folder
        for (dirpath,dirnames,filenames) in walk(path):
            f.extend(filenames)
            break
        
        if fw.isdigit() == False:
            #for firmware levels with bcd values, we convert them to hex
            fw = int(fw,16)         
        
        # keep a record of old firmware value
        current_fw = int(fw)            
        
        max_level = 0
        #iterate thru the list of firmware folders
        folder_str  =''
        for folder in dirnames:   
            folder_name = folder
            folder=self.parseFirmwareLevel(folder)
            
            # hold the string representation of the folder name
            folder_str = folder               
                         
            #compare the firmware of the device to the folder name 
            #corresponding to available firmware updates            
            if int(folder) > max_level:
                max_level = int(folder)
                
                #assign the new folder location
                new_fw_str = folder_str       
                name = folder_name
                #print "Found! FW Level:", new_fw_str
                TLOG.logger.info('TaskMgr::updatePendingDWUVideoTasks() - Found! FW package name:' + new_fw_str)
                TLOG.logger.info('TaskMgr::updatePendingDWUVideoTasks() - Found! FW package name:' + name)

        fw_path = (os.path.join(dirpath,name))
        
        controller = comp
        
        try:
            if (comp == DWU_COMPONENTS[0]) :
                fw_path = (os.path.join(fw_path,comp))
                tskcfg = Configs.UpdateTaskConfig(fw_path)
                mcu_fw = tskcfg.getFirmwareLevel()
                mcu_fw = mcu_fw[2:]
                
                #print int(mcu_fw)
                if ( int(mcu_fw) > current_fw) :
                    self.video_up_tasks.append( (task_name,fw_path,controller,) )       
            
            #FUTURE-Support: for Monitor Configurations Update
            elif (comp == DWU_COMPONENTS[1]) :
                #pass                          #do nothing when component is hw_cfg
                
                fw_path = (os.path.join(fw_path,comp))
                if os.path.exists(fw_path):
                    tskcfg = Configs.UpdateTaskConfig(fw_path)
                    mon_cfg = tskcfg.getFirmwareLevel()
                    mon_cfg = mon_cfg[2:]
                    
                    print int(mon_cfg)
                    if ( int(mon_cfg) > current_fw) :
                        self.video_up_tasks.append( (task_name,fw_path,controller,) )
                else:
                    pass      
                    
            elif (comp == DWU_COMPONENTS[2]) :
                fw_path = (os.path.join(fw_path,comp))
                tskcfg = Configs.UpdateTaskConfig(fw_path)
                scalar_fw = tskcfg.getFirmwareLevel()
                scalar_fw = scalar_fw[2:]
                
                #print int(scalar_fw)
                if ( int(scalar_fw) > current_fw ) :
                    self.video_up_tasks.append( (task_name,fw_path,controller,) )       
        
            elif (comp == DWU_COMPONENTS[3]) :
                fw_path = (os.path.join(fw_path,comp))
                tskcfg = Configs.UpdateTaskConfig(fw_path)
                cap_fw = tskcfg.getFirmwareLevel()
                cap_fw = cap_fw[2:]
                
                #print int(cap_fw)
                if (int(cap_fw) > current_fw) :
                    self.video_up_tasks.append( (task_name,fw_path,controller,) )
                
            else:
                print 'No Firmware Update Available!'  
                TLOG.logger.info('TaskMgr::updatePendingTaskList() - No Firmware Update Available!')
                                                                                              
        except ValueError:
            print 'Value Error'                                                               
            
#==========================================================================================================
# updatePendingTaskList() - update the PendingTasks list based on the firmware we got
#==========================================================================================================    
    
    def updatePendingTaskList(self,task_name,fw,controller,path):
        from os import walk
        f = []
        dirpath = ''
        dirnames = ''
        filenames = ''
        
        # get a list of all the folders inside the UpdateTask Folder
        for (dirpath,dirnames,filenames) in walk(path):
            f.extend(filenames)
            break
        
        if fw.isdigit() == False:
            #for firmware levels with bcd values, we convert them to hex
            fw = int(fw,16)         
        
        # keep a record of old firmware value
        new_fw = int(fw)            
        
        #iterate thru the list of firmware folders
        folder_str  =''
        for folder in dirnames:   
            folder_name = folder
            folder=self.parseFirmwareLevel(folder)
            
            # hold the string representation of the folder name
            folder_str = folder               
         
            if folder.isdigit() == False:
                # get the hex value of the folder name for comparison
                folder = int(folder, 16)        
                
            #compare the firmware of the device to the folder name 
            #corresponding to available firmware updates            
            if int(folder) > int(new_fw):
                new_fw = folder
                
                #assign the new folder location
                new_fw_str = folder_str       
                name = folder_name
                print "Found! FW Level:", new_fw_str
                TLOG.logger.info('TaskMgr::updatePendingTaskList() - Found! FW Level:' + new_fw_str)
                
        if ( new_fw > int(fw) ):
            fw_path = (os.path.join(dirpath,name))
            #self.pending_tasks.append( (task_type,fw_path) )
            if(task_name == 'SampoVideoFwFlashTask'):
                self.video_up_tasks.append( (task_name,fw_path,controller) )
            elif(task_name == 'EloTouchFwFlashTask'):
                self.touch_up_tasks.append( (task_name,fw_path,controller) )
        elif ( int(new_fw) == int(fw) ):
            print 'No Firmware Update Available!'
            TLOG.logger.info('TaskMgr::updatePendingTaskList() - No Firmware Update Available!')

#==========================================================================================================
# mergeTaskList() - Merge the Video and Touch Task Lists
#==========================================================================================================               
    def mergeTaskList(self):
        for vtask in self.video_up_tasks:
            task_type,path,controller = vtask
            if task_type in (item[0] for item in self.pending_tasks) and path in (item[1] for item in self.pending_tasks):
                pass
            else:
                self.pending_tasks.append(vtask)
        for ttask in self.touch_up_tasks:
            task_type,path,id = ttask
            
            #if task_type == 'OcularTouchFwFlashTask' :
            #    self.pending_tasks.append(ttask)
            if task_type in (item[0] for item in self.pending_tasks) and path in (item[1] for item in self.pending_tasks):
                pass
            else:
                self.pending_tasks.append(ttask)

#==========================================================================================================
# hasAvailableUpdateTasks() - Returns True if we have Update Tasks Available
#==========================================================================================================    
    def hasAvailableUpdateTasks(self):
        ret = False
               
        n = self.getMonitorList2()
        if (n != 0):
            
            for mon in self.mon_list:
                
                mon_type = mon.getMonType()
                touch_vid_pid = mon.getTouchVidPid()
                touch_fw = mon.getTouchFirmwareLevel()
                touch_serial = mon.getTouchSerialNumber()
                touch_devid = mon.getTouchDevId()
                
                
                video_vid_pid = mon.getVideoVidPid()
                video_fw = mon.getVideoFirmwareLevel()
                video_serial = mon.getVideoSerialNumber()
                video_devid  = mon.getVideoDevId()
                
                
                #check TouchUpdate
                self.checkForTouchFwUpdate(touch_vid_pid,touch_fw,touch_serial,mon_type,touch_devid)
                
                #check VideoUpdate
                self.checkForVideoFwUpdate(video_vid_pid,video_fw,video_serial,mon_type,video_devid)
                
        else:
            TLOG.logger.info('TaskMgr::hasAvailableUpdateTasks() -No Monitors Attached.')
            print 'No Monitors Attached'
            
        self.mergeTaskList()           
        count = len(self.pending_tasks)
        if (count != 0):
            ret = True;
                    
        return ret

#==========================================================================================================
# checkRecoveryFile() - function to check if recovery trigger file exist for the Sampo Recovery task
#==========================================================================================================    
    def checkRecoveryFile(self,filename): 
        
        ret = FILE_NOT_FOUND
        
        TLOG.logger.info('TaskMgr::checkRecoveryFile------------------START---------------')
        TLOG.logger.info('TaskMgr::checkRecoveryFile-Checking Recovery file...')
        if utils.doesFileExist(filename) == True:
            ret = FILE_FOUND
            #print 'TaskMgr::checkRecoveryFile-Found Sampo Recovery trigger file...'
            TLOG.logger.info('TaskMgr::checkRecoveryFile-Found Recovery file...')
        else:    
            print 'TaskMgr::checkRecoveryFile-No Recovery file...'
            TLOG.logger.info('TaskMgr::checkRecoveryFile-No Recovery file...')
                    
        TLOG.logger.info('TaskMgr::checkRecoveryFile-------------------END----------------\n')
        
        return ret
        


#==========================================================================================================
# isInSampoRecoveryMode() - Returns the sampo recovery mode status of the system
#==========================================================================================================    
    def isInSampoRecoveryMode(self):
        
        TLOG.logger.info('TaskMgr::isInSampoRecoveryMode------------------START---------------') 
        TLOG.logger.info('TaskMgr::isInSampoRecoveryMode-Checking if system is in sampo recovery mode...')
        recovery = False
        cmd = ''
        
        if ( self.checkRecoveryFile(AFTER_LOCAL) == FILE_FOUND ) :  
            TLOG.logger.info('TaskMgr::isInRecoveryMode()- Checking Sampo Recovery file.')
            #print  'Checking Sampo Recovery file.'  
            
            recovery_file = open(AFTER_LOCAL,'r+')
            lines = recovery_file.readlines()
            for line  in lines:
                #print line
                if(RECOVERY_STR in line):                  # We look for the string '4820MonUtil', this signifies that there is a pending recovery task
            
                    TLOG.logger.info('TaskMgr::isInRecoveryMode()- Found recovery String!- ' + line)
                    print 'Found Sampo Video Recovery file.' 
                    TLOG.logger.info('TaskMgr::isInRecoveryMode()- Proceed with Sampo Recovery Task')
                    print 'Proceed with Sampo Recovery Task'
                    
                    #ret = utils.runShell(str(line))
                    recovery = True
                    #recovery_type = 'Sampo'
                    
                    break
                
                else:
                    recovery = False
            
            recovery_file.close()
            
            self.rcv_cmd = line
        else:
                recovery =  False
                TLOG.logger.info('TaskMgr::isInSamopRecoveryMode()-FALSE')
                print 'Not in Sampo Recovery Mode...'
        
        return recovery

#==========================================================================================================
# isInEloRecoveryMode() - Returns the elo recovery mode status of the system
#==========================================================================================================       
    def isInEloRecoveryMode(self):   
          
        TLOG.logger.info('TaskMgr::isInEloRecoveryMode------------------START---------------') 
        TLOG.logger.info('TaskMgr::isInEloRecoveryMode-Checking if system is in elo recovery mode...')
        recovery = False
        cmd = ''
        
        if( self.checkRecoveryFile(ELO_RECOVERY_FILE) == FILE_FOUND ) :                  
                TLOG.logger.info('TaskMgr::isInRecoveryMode()- Checking Elo Recovery file.')
                #print  'Checking Elo Touch Recovery file.'
               
                recovery_file = open(ELO_RECOVERY_FILE,'r+')
                lines = recovery_file.readlines()
                for line  in lines:
                #print line
                    if('EloRecover' in line):                  
                        
                        line = line.strip('\n')
                        tsk,cmd = line.split(' ')
                        
                        TLOG.logger.info('TaskMgr::isInRecoveryMode()- Found recovery String!- ' + line ) 
                        print 'Found Elo Touch Recovery file.'
                        TLOG.logger.info('TaskMgr::isInRecoveryMode()- Proceed with Elo Recovery Task')
                        print 'Proceed with Elo Recovery Task'
                        
                        recovery = True
                        break
                    
                recovery_file.close()
                TLOG.logger.info('TaskMgr::isInRecoveryMode - EloRecovery --------\n') 
                
                self.rcv_cmd = cmd 
        else:
                recovery =  False
                TLOG.logger.info('TaskMgr::isInEloRecoveryMode()-FALSE')
                print 'Not in Elo Recovery Mode...'
            
    
        return recovery
    
        
#==========================================================================================================
# isInAipRecoveryMode() - Returns the aipfwupdate recovery mode status of the system
#==========================================================================================================       
    def isInAipRecoveryMode(self):   
          
        TLOG.logger.info('TaskMgr::isInAipRecoveryMode------------------START---------------') 
        TLOG.logger.info('TaskMgr::isInAipRecoveryMode-Checking if system is in aip recovery mode...')
        recovery = False
        path = ''
        fw = ''
        
        if( self.checkRecoveryFile(AIP_RECOVERY_FILE) == FILE_FOUND ) :                  
                TLOG.logger.info('TaskMgr::isInRecoveryMode()- Checking Aip Recovery file.')
                
                recovery_file = open(AIP_RECOVERY_FILE,'r+')
                lines = recovery_file.readlines()
                for line  in lines:
                #print line
                    if('AIPRecover' in line):                  
                        
                        line = line.strip('\n')
                        tsk,path,fw = line.split(',')
                        
                        TLOG.logger.info('TaskMgr::isInAipRecoveryMode()- Found recovery String!- ' + line ) 
                        print 'Found Aipfwupdate Video Recovery file.'
                        TLOG.logger.info('TaskMgr::isInAipRecoveryMode()- Proceed with Aip Recovery Task')
                        print 'Proceed with Aip Recovery Task'
                        
                        recovery = True
                        break
                    
                recovery_file.close()
                TLOG.logger.info('TaskMgr::isInRecoveryMode - AipRecovery --------\n') 
                
                self.path = path
                self.fw_name = fw 
        else:
                recovery =  False
                TLOG.logger.info('TaskMgr::isInAipRecoveryMode()-FALSE')
                print 'Not in Aip Recovery Mode...'
            
    
        return recovery
        
#==========================================================================================================
# isInOcularRecoveryMode() - Returns the ocular recovery mode status of the system
#==========================================================================================================       
    def isInOcularRecoveryMode(self):   
          
        TLOG.logger.info('TaskMgr::isInOcularRecoveryMode------------------START---------------') 
        TLOG.logger.info('TaskMgr::isInOcularRecoveryMode-Checking if system is in ocular recovery mode...')
        recovery = False
        path = ''
        fw = ''
        
        if( self.checkRecoveryFile(OCULAR_RECOVERY_FILE) == FILE_FOUND ) :                  
                TLOG.logger.info('TaskMgr::isInOcularRecoveryMode()- Checking Ocular Recovery file.')
                
                recovery_file = open(OCULAR_RECOVERY_FILE,'r+')
                lines = recovery_file.readlines()
                for line  in lines:
                #print line
                    if('OcularRecover' in line):                  
                        
                        line = line.strip('\n')
                        tsk,path,conf_path = line.split(',')
                        
                        TLOG.logger.info('TaskMgr::isInOcularRecoveryMode()- Found recovery String!- ' + line ) 
                        print 'Found Ocular Touch Recovery file.'
                        TLOG.logger.info('TaskMgr::isInOcularRecoveryMode()- Proceed with Ocular Recovery Task')
                        print 'Proceed with Ocular Recovery Task'
                        
                        recovery = True
                        break
                    
                recovery_file.close()
                TLOG.logger.info('TaskMgr::isInOcularRecoveryMode - OcularRecovery --------\n') 
                
                self.path = path
                self.fw_name = fw 
                self.conf_path = conf_path
        else:
                recovery =  False
                TLOG.logger.info('TaskMgr::isInOcularRecoveryMode()-FALSE')
                print 'Not in Ocular Recovery Mode...'
            
    
        return recovery
    
        
#==========================================================================================================
# performTasks() - The main function to be called by the MUS App/ Service to perform the actual update task
#==========================================================================================================    
    def performTasks(self):
        
        '''
            performTasks Sequence of Tasks
                     
            1. check if in Recovery mode (Sampo , ELO)
            2. check for available update tasks
            3. perform necessary update tasks
            
        ''' 
        
        TLOG.logger.info('TaskMgr::performTasks()=============================================================================================== S T A R T \n')       
        
        #recovery check
             
        if  self.isInEloRecoveryMode() == True:
            TLOG.logger.info('TaskMgr::performTasks()- Calling doEloTouchFwFlashRecoveryTask()')
            print 'Please wait...Performing Touch Firmware Recovery, the System will reboot after the recovery operation... '
           
            self.doEloTouchFwFlashRecoveryTask(self.rcv_cmd)
            utils.runShell('reboot')
            sys.exit(0)
            
        if  self.isInAipRecoveryMode() == True:
            TLOG.logger.info('TaskMgr::performTasks()- Calling doAipVideoFwFlashRecoveryTask()')
            print 'Please wait...Performing Video Firmware Recovery, the System will reboot after the recovery operation... '
           
            self.doAipVideoFwFlashRecoveryTask(self.path,self.fw_name)
            utils.runShell('reboot')
            sys.exit(0)
        
        if  self.isInOcularRecoveryMode() == True:
            TLOG.logger.info('TaskMgr::performTasks()- Calling doOcularTouchFwFlashRecoveryTask()')
            print 'Please wait...Performing Touch Firmware Recovery, the System will reboot after the recovery operation... '
           
            self.doOcularTouchFwFlashRecoveryTask(self.path,self.conf_path)
            utils.runShell('reboot')
            sys.exit(0)
            
        elif self.isInSampoRecoveryMode() == True:
            TLOG.logger.info('TaskMgr::performTasks()- Calling doSampoVideoFwFlashRecoveryTask()')
            print 'Please wait...Performing Video Firmware Recovery, the System will reboot after the recovery operation... '
           
            self.doSampoVideoFwFlashRecoveryTask(self.rcv_cmd)
            utils.runShell('reboot')
            sys.exit(0)
                       
      
        #elif self.haveAvailableUpdateTasks() == True :
        elif self.hasAvailableUpdateTasks() == True :
            
            TLOG.logger.info('TaskMgr::performTasks()- Found Available Update tasks...')
            
            for task in self.pending_tasks:
                
                tskname,up_path,controller = task
                TLOG.logger.info('TaskMgr::performTasks() -Task : '+ tskname)
                
                TLOG.logger.info('TaskMgr::performTasks() -Calling MUSConfigParser() ')
                
                update_config = Configs.UpdateTaskConfig(up_path)
                           
                task_type = update_config.getTaskType()              
                self.fw_path = update_config.getFirmwarePath()
                self.fw_name = update_config.getFirmwareName()
                self.controller = controller
                
                TLOG.logger.info('TaskMgr::performTasks()- TaskName= '+ task_type)
                print 'TaskMgr::performTasks()- TaskName= '+ task_type
                
                #4820 Monitors Video Firmware Update Tasks
                if (tskname.strip('\n') == 'SampoVideoFwFlashTask'):
                    TLOG.logger.info('TaskMgr::performTasks()- Calling doSampoVideoFwFlashTask()')
                    
                    exit_status = self.doSampoVideoFwFlashTask(task_type)          
                    TLOG.logger.info('TaskMgr::performTasks()-'+ ' Exit Status is:' + str(exit_status) )
                    
                    if exit_status != 0:
                        TLOG.logger.info('TaskMgr::performTasks()-SampoVideoFlashTask Failed!-The System will undergo reboot to recover.' )
                        #utils.updateBootCount()
                        print 'Update task failed...The System will undergo reboot to recover.' 
                        utils.runShell('reboot')
                        raise RuntimeError("SampoVideoFlashTask Task Failed!")
                
                #4820 Monitors Touch Firmware Update Tasks
                elif (tskname.strip('\n') == 'EloTouchFwFlashTask'):
                    TLOG.logger.info('TaskMgr::performTasks()- Calling doEloTouchFlashTask()')
                    
                    exit_status = self.doEloTouchFwFlashTask(task_type)   
                    TLOG.logger.info('TaskMgr::performTasks()-'+ ' Exit Status is:' + str(exit_status) )
                    
                    # FLIWA_15052014   Add reboot After every firmware flash for 4820 Monitors
                    # FLIWA_15052014   Workaround when there is a 6149 Monitors around
#                    if exit_status == 0:
#                        utils.runShell('rm ' + ELO_RECOVERY_FILE)
#                        utils.runShell('reboot')
                            
                    if exit_status != 0:
                        TLOG.logger.info('TaskMgr::performTasks()-EloTouchFwFlashTask Failed!-The System will undergo reboot to recover.' )
                        #utils.updateBootCount()
                        time.sleep(1)
                        print 'Update task failed...The System will undergo reboot to recover.'                      
                        utils.runShell('reboot')  
                        raise RuntimeError("EloTouchFwFlashTask Task Failed!")
                
                #6149 Monitors Touch Firmware Update Tasks    
                elif (tskname.strip('\n') == 'OcularTouchFwFlashTask'):
                    TLOG.logger.info('TaskMgr::performTasks()- Calling doOcularTouchFlashTask()')
                    
                    exit_status = self.doOcularTouchFwFlashTask(task_type)   
                    TLOG.logger.info('TaskMgr::performTasks()-'+ ' Exit Status is:' + str(exit_status) )
                    
                    if exit_status != 0:
                        TLOG.logger.info('TaskMgr::performTasks()-OcularTouchFwFlashTask Failed!-The System will undergo reboot to recover.' )
                        #utils.updateBootCount()
                        time.sleep(1)
                        print 'Update task failed...The System will undergo reboot to recover.'                      
                        utils.runShell('reboot')  
                        raise RuntimeError("OcularTouchFwFlashTask Task Failed!")
                    
                #6149 Monitors Touch Config Update Tasks    
                elif (tskname.strip('\n') == 'OcularTouchXcfgLoadTask'):
                    TLOG.logger.info('TaskMgr::performTasks()- Calling doOcularTouchFlashTask()')
                    
                    task_type = UPDATE_TYPES[7]
                    exit_status = self.doOcularTouchFwFlashTask(task_type)   
                    TLOG.logger.info('TaskMgr::performTasks()-'+ ' Exit Status is:' + str(exit_status) )
                    
                    if exit_status != 0:
                        TLOG.logger.info('TaskMgr::performTasks()-OcularTouchFwFlashTask Failed!-The System will undergo reboot to recover.' )
                        #utils.updateBootCount()
                        time.sleep(1)
                        print 'Update task failed...The System will undergo reboot to recover.'                      
                        utils.runShell('reboot')  
                        raise RuntimeError("OcularTouchFwFlashTask Task Failed!")    
                
                #6149 Monitors Video Firmware Update Tasks
                elif (tskname.strip('\n') == 'WistronVideoFwFlashTask'):
                    TLOG.logger.info('TaskMgr::performTasks()- Calling doWistronVideoFwFlashTask()')
                    
                    exit_status = self.doWistronVideoFwFlashTask(task_type)   
                    TLOG.logger.info('TaskMgr::performTasks()-'+ ' Exit Status is:' + str(exit_status) )
                    
                    if exit_status != 0:
                        TLOG.logger.info('TaskMgr::performTasks()-WistronVideoFwFlashTask Failed!-The System will undergo reboot to recover.' )
                        #utils.updateBootCount()
                        time.sleep(1)
                        print 'Update task failed...The System will undergo reboot to recover.'                      
                        utils.runShell('reboot')  
                        raise RuntimeError("WistronVideoFwFlashTask Task Failed!")
#                 
#                
                                                                             
        else: #performtask
                #We don't have task available, so just exit from the method and kill the service.
                TLOG.logger.info('TaskMgr::performTasks()- No Firmware Update task done.')    
                print  'No Firmware Update task done.'   
            
            
        if (  ( len(self.mon_list) != 0 )  and  ( utils.doesFileExist(ELO_INI) == True ) and ( self.HAS_4820_MON != False) ) :
            TLOG.logger.info('TaskMgr::performTasks()- Calling doEloConfigTask()')
            exit_status = self.doEloConfigTask(ELO_INI)
            if exit_status != 0:
                raise RuntimeError("EloConfigTask Task Failed!-exiting :" )
          
            #call rstelo
            self.doRstElo()  
        else:
            TLOG.logger.info('TaskMgr::performTasks()-EloConfigTask is not performed because either No 4820-x1x/xLx Monitor Attached or Touch Configuration file is not found.' )
            print  'EloConfigTask is not performed because either No 4820-x1x/xLx Monitor Attached or Touch Configuration file is not found.'
    
        TLOG.logger.info('TaskMgr::performTasks()================================================================================================== E N D \n')            
            
        return EXIT_SUCCESS   #will always return success with or without a task!
    
    
            
#==========================================================================================================
# doSampoVideoFwFlashRecoveryTask() -  function to execute SampoVideoFlashRecoveryTask run method
#==========================================================================================================
    def doSampoVideoFwFlashRecoveryTask(self,cmd):
        
        svfrt = sampo.VideoFwRecoveryFlashTask()       
        svfrt.run(cmd)     

#==========================================================================================================
# doSampoVideoFwFlashRecoveryTask() -  function to execute SampoVideoFlashRecoveryTask run method
#==========================================================================================================
    def doEloTouchFwFlashRecoveryTask(self,fw_path):
        
        etfrt = elo.TouchFwRecoveryFlashTask()       
        etfrt.run(fw_path)     
        
#==========================================================================================================
# doAipVideoFwFlashRecoveryTask() -  function to execute AipVideoFlashRecoveryTask run method
#==========================================================================================================
    def doAipVideoFwFlashRecoveryTask(self,fw_path,name):
        
        wvfrt = wistron.VideoFwRecoveryFlashTask(name)       
        wvfrt.run(fw_path)     

#==========================================================================================================
# doAipVideoFwFlashRecoveryTask() -  function to execute AipVideoFlashRecoveryTask run method
#==========================================================================================================
    def doOcularTouchFwFlashRecoveryTask(self,fw_path,conf):
        
        wvfrt = ocular.TouchFwRecoveryFlashTask()       
        wvfrt.run(fw_path,conf)     


#==========================================================================================================
# doSampoVideoFwFlashTask() -  function to execute SampoVideoFlashTask run method
#==========================================================================================================
    def doSampoVideoFwFlashTask(self,tsktype):
        
        result = EXIT_SUCCESS
        if (tsktype == UPDATE_TYPES[0]):
           
            svft = sampo.VideoFwFlashTask_4820_21x51x(self.fw_path,self.controller)      
            result = svft.run()      
             
        elif (tsktype == UPDATE_TYPES[1]):     
            svft = sampo.VideoFwFlashTask_4820_2Lx5Lx(self.fw_path,self.controller)      
            result = svft.run()      
        
        
        return result
    
    
#==========================================================================================================
# doEloTouchFwFlashTask() -  function to execute Fw Flash Command
#==========================================================================================================        
    def doEloTouchFwFlashTask(self,tsktype):
        
        exit_status = 0;
        
        if (tsktype == UPDATE_TYPES[2]):
            eyft = elo.TouchFwFlashTask_21x51x(self.fw_path,self.fw_name,self.controller)       
            exit_status  = eyft.run()    

        elif (tsktype == UPDATE_TYPES[3]):
            eaft = elo.TouchFwFlashTask_2Lx5Lx(self.fw_path,self.fw_name,self.controller)      
            exit_status = eaft.run()
             
        return exit_status


#==========================================================================================================
# doEloConfigTask() -  function to execute EloConfigTask run method
#==========================================================================================================    
    def doEloConfigTask(self,config_path):
        
        eCt = elo.ConfigTask()      
        result = eCt.run(config_path)
        return result


###MUS 2.0 -  Added by fliwa###

#==========================================================================================================
# doOcularTouchFwFlashTask() -  function to execute Ocular Touch Fw Flash Command
#==========================================================================================================        
    def doOcularTouchFwFlashTask(self,tsktype):
        
        exit_status = 0;
        
        if (tsktype == UPDATE_TYPES[6]):
            odft = ocular.TouchFwFlashTask(self.fw_path,self.fw_name,self.controller)       
            exit_status  = odft.run()
    
        #Load XCFG only
        elif (tsktype == UPDATE_TYPES[7]):
            odft = ocular.TouchFwFlashTask(self.fw_path,self.fw_name,self.controller,True)       
            exit_status  = odft.run()  
                  
        else:
            print 'Error: Unknown Type'
             
        return exit_status



#==========================================================================================================
# doWistronVideoFwFlashTask() -  function to execute WistronVideoFlashTask run method
#==========================================================================================================
    def doWistronVideoFwFlashTask(self,tsktype):
        
        result = EXIT_SUCCESS
        if (tsktype == UPDATE_TYPES[5]):
           
            wvft = wistron.VideoFwFlashTask_6149_DWU(self.fw_path,self.fw_name,self.controller)      
            result = wvft.run()      
            
        return result
           
#==========================================================================================================
# getDeviceState() - function to get the current state of the touch device
#==========================================================================================================
    def getDeviceState(self,device_ ):
        '''
        Checking device state
        '''
        state = UNKNOWN
        deviceFound = False
        
        TLOG.logger.info('TaskMgr::getDeviceState-----------------START---------------')
                  
        if utils.doesFileExist(ELO_DOWNLOAD) != True:
                       
            print 'TaskMgr::getDeviceState()-No ELO Download script found!'
            TLOG.logger.error('TaskMgr::getDeviceState()-No ELO Download script found!')
            raise IOError('No ELO Download script found!')
        
        else:
            
            deviceFound = self.find_devices_in_fousb_mode( device_ )
            if (deviceFound == True):
                #Need to recover state of the device first:
                print  'TaskMgr::getDeviceState() :' + device_ + 'monitor needs to be recovered'
                TLOG.logger.info('TaskMgr::getDeviceState() :' + device_ + 'monitor needs to be recovered')
                state = -2
            else:
                #try to call inventory to see if any such devices are attached.        
                result = self.have_attached_display_types(device_ )
                
                if (result == NO_MONITORS_ATTACHED):
                    print  'TaskMgr::getDeviceState() :No Displays are attached at all!'
                    TLOG.logger.info('TaskMgr::getDeviceState() :No Displays are attached at all!')
                    state = -1
                    
                elif (result == True):
                    print  'TaskMgr::getDeviceState() : Has attached ' + device_ + 'display'
                    TLOG.logger.info('TaskMgr::getDeviceState() : has attached ' + device_ + 'display')
                    state = 1 
                                   
                elif (result == False):
                    print  'TaskMgr::getDeviceState() : No attached ' + device_ + 'display'
                    TLOG.logger.info('TaskMgr::getDeviceState() : No attached ' + device_ + 'display')
                    state = 2
    
        print 'TaskMgr::getDeviceState() - END '  
        TLOG.logger.info('TaskMgr::getDeviceState() -----------------END-----------------\n')
       
        return state

#==========================================================================================================
# find_devices_in_fousb_mode() - 
#==========================================================================================================
    def find_devices_in_fousb_mode(self,device_):
     
        screen = ''
        controller = ''
        result = ''
        model=''
        foundDeviceType = False
        
        find_cmd = 'elo_download -find' 
        process = subprocess.Popen(find_cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE) 
        while True:
            output_line = process.stdout.readline()
            #print output_line
            if not output_line:
                break                    
            #Response line
            if "RESPONSE" in output_line:               
                res = output_line.split()
                for r in res:
                    #print r                
                    if "SCREEN" in r:
                        idx = r.find("=")
                        for c in r[idx + 1:len(r)]:
                            if c == '\"':
                                continue
                            screen += c
                        print screen
                        TLOG.logger.info('TaskMgr::find_devices_in_fousb_mode() :screen =' + screen)
                    elif "CONTROLLER" in r:
                        idx = r.find("=")
                        for c in r[idx + 1:len(r)]:
                            if  c == '\"':
                                continue
                            controller += c
                        print controller
                        TLOG.logger.info('TaskMgr::find_devices_in_fousb_mode() :controller =' + controller)
                    elif "MODEL" in r:
                        idx = r.find("=")
                        for c in r[idx + 1:len(r)]:
                            if c == '\"' or c == '>' :
                                continue
                            model += c
                        print model
                        TLOG.logger.info('TaskMgr::find_devices_in_fousb_mode() :model =' + model)
                    else :
                        idx = r.find("=")
                        for c in r[idx + 1:len(r)]:
                            if c == '\"':
                                continue
                            result += c
                            #print result                   
            else:
                #print 'find_devices_in_fousb_mode() - No Displays in FOUSB MODE'
                TLOG.logger.info('TaskMgr::find_devices_in_fousb_mode() :- No Displays in FOUSB MODE')
    
            if device_ == model:
                print device_ + " = " + model
                foundDeviceType = True
                break
            else:
                screen=''
                controller=''
                model=''
                TLOG.logger.info('TaskMgr::find_devices_in_fousb_mode() :- No Displays in FOUSB MODE')
    
            process.wait()
    
        return foundDeviceType
         
     
#==========================================================================================================
# have_attached_display_types() - 
#==========================================================================================================
    def have_attached_display_types(self,device_ ):   
        screen = ''
        controller = ''
        result = ''
        firmware = ''
        model = '4820-'
        foundDisplayType = False
     
        elo_inventory_cmd = 'elo_download -inventory'     
        process = subprocess.Popen(elo_inventory_cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        while True:
            output_line = process.stdout.readline()
            #print output_line
            if not output_line:
                break
            #Response line
            if "RESPONSE" in output_line:
                #print output_line
                if "ERROR" in output_line:
                    print  'TaskMgr::getDeviceState() - ERROR: Controller not detected'
                    TLOG.logger.error('TaskMgr::getDeviceState() ERROR: Controller not detected')
                    return NO_MONITORS_ATTACHED
                else :
                    res = output_line.split()
                    for r in res:
                        #print r
                        if "SCREEN" in r:       
                            idx = r.find("=")
                            for c in r[idx + 1:len(r)]:
                                if c == '\"':
                                    continue
                                screen += c
                            print screen
                            TLOG.logger.info('TaskMgr::getDeviceState() :screen =' + screen)
                        elif "CONTROLLER" in r:
                            idx = r.find("=")
                            for c in r[idx + 1:len(r)]:
                                if  c == '\"':
                                    continue
                                controller += c
                            #print controller
                            TLOG.logger.info('TaskMgr::getDeviceState() :controller =' + controller)
                        #elif "MODEL" in r:
                        elif "4820" in r:
                            idx = r.find("-")
                            for c in r[idx + 1:len(r)]:
                                if c == '\"' or c == '>' :
                                    continue
                                model += c
                            #print model
                            TLOG.logger.info('TaskMgr::getDeviceState() :model =' + model)
                        else :
                            idx = r.find("=")
                            for c in r[idx + 1:len(r)]:
                                if c == '\"':
                                    continue
                                result += c
                                #print result
            elif "DATA" in output_line:
                idx = output_line.find(">")
                for c in output_line[idx + 1:len(output_line)]:
                    if c == '<':
                        break
                    firmware += c
                #print firmware 
                TLOG.logger.info('TaskMgr::getDeviceState() :firmware =' + firmware)
                
            if device_ == model:
                print device_ + " = " + model
                foundDisplayType = True                  
                break
            else:
                screen = ''
                controller = ''
                result = ''
                firmware = ''
                model = '4820-'
            
            process.wait()

        return foundDisplayType 
    

#==========================================================================================================
'''
TouchMonitor - abstraction of the touch monitor 
'''
#==========================================================================================================

class TouchMonitor(object):
    def __init__(self):
        '''
        Constructor
        '''
               
        #TLOG.logger.info('TouchMonitor Constructed')
       
        self.screenNumber = ''
    
    #touch components
        self.touch_serial = ''
        self.touchFirmwareLevel = ''
        self.touch_vidpid = ''
        self.touch_dev_id = ''
    
    #video components
        self.video_serial =''        
        self.videoFirmwareLevel = ''
        self.video_vidpid = ''
        self.video_dev_id = ''
        
        self.mon_type =''
     
        
               
    def setMonType(self,montype):
        self.mon_type = montype
    
    def setScreenNo(self,scrn_no):
        self.screenNumber = scrn_no

    
    def setVideoSerialNumber(self,serial):
        self.video_serial = serial
    
    def setVideoVidPid(self,vidpid):
        self.video_vidpid = vidpid

    def setVideoDevId(self,dev_id):
        self.video_dev_id = dev_id
        
    def setVideoFwLevel(self,fwlevel):
        self.videoFirmwareLevel = fwlevel

    
    
    def setTouchSerialNumber(self,serial):
        self.touch_serial = serial

    def setTouchVidPid(self,vidpid):
        self.touch_vidpid = vidpid
        
    def setTouchDevId(self,dev_id):
        self.touch_dev_id = dev_id
    
    def setTouchFwLevel(self,fwlevel):
        self.touchFirmwareLevel = fwlevel
        
    
    def getMonType(self):
        
        TLOG.logger.info('TouchMonitor::getScreenNumber() - screen# = ' + self.mon_type )
        
        return self.mon_type
                     
    def getTouchVidPid(self):
        
         
        TLOG.logger.info('TouchMonitor::getTouchVidPid() - vidpid = ' + self.touch_vidpid )
        
        return self.touch_vidpid

    def getVideoVidPid(self):
        
         
        TLOG.logger.info('TouchMonitor::getVideoVidPid() - vidpid = ' + self.video_vidpid  )
        
        return self.video_vidpid
    
    def getTouchDevId(self):
        
         
        TLOG.logger.info('TouchMonitor::getTouchDevId() - dev id = ' + self.touch_dev_id )
        
        return self.touch_dev_id

    def getVideoDevId(self):
        
         
        TLOG.logger.info('TouchMonitor::getVideoDevId() - dev id = ' + self.video_dev_id  )
        
        return self.video_dev_id

    
    def getVideoSerialNumber(self):
        
            
        TLOG.logger.info('TouchMonitor::getControllerId() - controller id = ' + self.video_serial )
        
        return self.video_serial

    def getTouchSerialNumber(self):
        
            
        TLOG.logger.info('TouchMonitor::getControllerId() - controller id = ' + self.touch_serial )
        
        return self.touch_serial
   

    
    def getTouchFirmwareLevel(self):
        
        TLOG.logger.info('TouchMonitor::getFirmwareLevel() - fw_level = ' + self.touchFirmwareLevel  )
        
        return self.touchFirmwareLevel 
    
    def getVideoFirmwareLevel(self):
        
        TLOG.logger.info('TouchMonitor::getFirmwareLevel() - fw_level = ' + self.videoFirmwareLevel  )
        
        return self.videoFirmwareLevel

    def getScreenNumber(self):
        
        TLOG.logger.info('TouchMonitor::getScreenNumber() - screen# = ' + self.screenNumber )
        
        return self.screenNumber
    
    def printInfo(self):
        print 'screen# = ' + self.screenNumber

        print 'touch serial# =' + self.touch_serial    
        print 'touch controllerId =' + self.touch_vidpid
        print 'touch firmware = ' + self.touchFirmwareLevel
    
        print 'video serial# =' + self.video_serial    
        print 'video controllerId =' + self.video_vidpid      
        print 'video firmware = ' + self.videoFirmwareLevel
